import tg
from formencode import Invalid, FancyValidator

try:
    import sqlalchemy

    class SQLAEntityConverter(FancyValidator):
        def __init__(self, klass, session=None, slugified=False):
            super(FancyValidator, self).__init__(not_empty=True)
            self.klass = klass
            self.session = session
            self.slugified = slugified

        def _to_python(self, value, state):
            if self.session is None:
                self.session = tg.config['DBSession']

            if self.slugified:
                try:
                    value = value.rsplit('-', 1)[-1]
                except:
                    value = value

            return self.session.query(self.klass).get(value)

        def validate_python(self, value, state):
            if not value:
                raise Invalid('object not found', value, state)

except ImportError: #pragma: no cover
    pass


try:
    import ming
    from bson import ObjectId

    class MingEntityConverter(FancyValidator):
        def __init__(self, klass, slugified=False):
            super(FancyValidator, self).__init__(not_empty=True)
            self.klass = klass
            self.slugified = slugified

        def _to_python(self, value, state):
            if self.slugified:
                try:
                    value = value.rsplit('-', 1)[-1]
                except:
                    value = value

            try:
                obj_id = ObjectId(value)
            except:
                raise Invalid('object not found', value, state)
            
            return self.klass.query.get(_id=obj_id)

        def validate_python(self, value, state):
            if not value:
                raise Invalid('object not found', value, state)

except ImportError: #pragma: no cover
    pass


from formencode import Invalid
from nose.tools import raises
from sqla_base import setup_database, clear_database, DBSession, Thing
from tgext.datahelpers.validators import SQLAEntityConverter

def setup():
    setup_database()

class TestSQLAEntityConverter(object):
    def __init__(self):
        self.validator = SQLAEntityConverter(Thing, DBSession)
        self.slug_validator = SQLAEntityConverter(Thing, DBSession, slugified=True)

    def setup(self):
        clear_database()

    def test_found_entity(self):
        DBSession.add(Thing(uid=2, name=u'Foo'))
        DBSession.add(Thing(uid=1, name=u'Bar'))
        DBSession.flush()

        x = self.validator.to_python(1)
        assert x
        assert x.name == u'Bar'

    @raises(Invalid)
    def test_notfound_entity(self):
        DBSession.add(Thing(uid=1, name=u'Bar'))
        DBSession.flush()

        x = self.validator.to_python(5)

    @raises(Invalid)
    def test_missing_entity(self):
        self.validator.to_python(None)

    def test_string_id(self):
        DBSession.add(Thing(uid=1, name=u'Bar'))
        DBSession.flush()

        x = self.validator.to_python('1')
        assert x
        assert x.name == u'Bar'

    @raises(Invalid)
    def test_not_an_id(self):
        DBSession.add(Thing(uid=1, name=u'Bar'))
        DBSession.flush()

        x = self.validator.to_python('asd')

    def test_found_with_slug(self):
        DBSession.add(Thing(uid=2, name=u'Foo'))
        DBSession.add(Thing(uid=1, name=u'Bar'))
        DBSession.flush()

        x = self.slug_validator.to_python('foo-1')
        assert x
        assert x.name == u'Bar'

    def test_found_slugified_without_slug(self):
        DBSession.add(Thing(uid=2, name=u'Foo'))
        DBSession.add(Thing(uid=1, name=u'Bar'))
        DBSession.flush()

        x = self.slug_validator.to_python('1')
        assert x
        assert x.name == u'Bar'

    def test_found_slugified_on_int(self):
        DBSession.add(Thing(uid=2, name=u'Foo'))
        DBSession.add(Thing(uid=1, name=u'Bar'))
        DBSession.flush()

        x = self.slug_validator.to_python(1)
        assert x
        assert x.name == u'Bar'



import ming
from ming.datastore import DataStore
from ming import Session
from ming import schema as s
from ming.orm import ThreadLocalORMSession, FieldProperty, Mapper
from ming.orm.declarative import MappedClass
from datetime import datetime

mainsession = Session()
DBSession = ThreadLocalORMSession(mainsession)

database_setup = False
bind = None

def setup_database():
    global bind, database_setup
    if not database_setup:
        bind = DataStore('mim:///', database='test')
        mainsession.bind = bind
        ming.orm.Mapper.compile_all()
        database_setup = True

def clear_database():
    global engine, database_setup
    if not database_setup:
        setup_database()

class Thing(MappedClass):
    class __mongometa__:
        session = DBSession
        name = 'thing'

    _id = FieldProperty(s.ObjectId)
    name = FieldProperty(s.String)

class ThingWithDate(MappedClass):
    class __mongometa__:
        session = DBSession
        name = 'thing'

    _id = FieldProperty(s.ObjectId)
    name = FieldProperty(s.String)
    updated_at = FieldProperty(s.DateTime, if_missing=datetime.now)



# -*- coding: utf-8 -*-
from tg.exceptions import HTTPNotFound
from nose.tools import raises
from sqla_base import setup_database, clear_database, DBSession, Thing

def setup():
    setup_database()

from tgext.datahelpers.utils import slugify, fail_with

class TestSlugify(object):
    def setup(self):
        clear_database()

        DBSession.add(Thing(uid=2, name=u'Foo'))
        DBSession.add(Thing(uid=1, name=u'Bar'))
        DBSession.flush()

    def test_slugify(self):
        thing = DBSession.query(Thing).get(1)
        slug = slugify(thing, thing.name)
        assert slug == 'bar-1', slug

    def test_slugify_unicode(self):
        thing = DBSession.query(Thing).get(2)
        slug = slugify(thing, u'This is a very long àèìòù phrase ')
        assert slug == 'this-is-a-very-long-aeiou-phrase-2', slug

    def test_slugify_unicode_ignore(self):
        thing = DBSession.query(Thing).get(2)
        slug = slugify(thing, u'This is a very long ’”««¡≠€∂ß phrase ')
        assert slug == 'this-is-a-very-long-phrase-2', slug

class TestFailWith(object):
    @raises(HTTPNotFound)
    def test_fail_with(self):
        fail_with(404)()

    @raises(HTTPNotFound)
    def test_fail_with_args(self):
        fail_with(404)(1,2,3, x=5)


from sqlalchemy import *
from sqlalchemy.orm import scoped_session, sessionmaker, relation
from sqlalchemy.ext.declarative import declarative_base
from datetime import datetime

maker = sessionmaker(autoflush=True, autocommit=False)
DBSession = scoped_session(maker)
DeclarativeBase = declarative_base()
metadata = DeclarativeBase.metadata

database_setup=False
engine = None

def setup_database():
    global engine, database_setup
    if not database_setup:
        engine = create_engine('sqlite:///:memory:')
        DBSession.configure(bind=engine)
        database_setup = True

def clear_database():
    global engine, database_setup
    if not database_setup:
        setup_database()

    DBSession.rollback()
    DeclarativeBase.metadata.drop_all(engine)
    DeclarativeBase.metadata.create_all(engine)

class Thing(DeclarativeBase):
    __tablename__ = 'thing'

    uid = Column(Integer, autoincrement=True, primary_key=True)
    name = Column(Unicode(16), unique=True)

class ThingWithDate(DeclarativeBase):
    __tablename__ = 'thing_with_date'

    uid = Column(Integer, autoincrement=True, primary_key=True)
    name = Column(Unicode(16), unique=True)
    updated_at = Column(DateTime, default=datetime.now)

    related_thing_id = Column(Integer, ForeignKey('thing.uid'))
    related_thing = relation(Thing)



from bson import ObjectId
from formencode import Invalid
from nose.tools import raises
from ming_base import setup_database, clear_database, DBSession, Thing
from tgext.datahelpers.validators import MingEntityConverter

def setup():
    setup_database()

class TestMingEntityConverter(object):
    def __init__(self):
        self.validator = MingEntityConverter(Thing)
        self.slug_validator = MingEntityConverter(Thing, slugified=True)

    def setup(self):
        clear_database()

    def test_found_entity(self):
        t1 = Thing(name=u'Foo')
        t2 = Thing(name=u'Bar')
        DBSession.flush()
        DBSession.close()

        x = self.validator.to_python(t2._id)
        assert x
        assert x.name == u'Bar'

    @raises(Invalid)
    def test_notfound_entity(self):
        t1 = Thing(name=u'Bar')
        DBSession.flush()
        DBSession.close()

        x = self.validator.to_python(ObjectId('000000000000000000000000'))

    def test_string_id(self):
        t1 = Thing(name=u'Bar')
        DBSession.flush()
        DBSession.close()

        x = self.validator.to_python(str(t1._id))
        assert x
        assert x.name == u'Bar'

    @raises(Invalid)
    def test_not_an_id(self):
        t1 = Thing(name=u'Bar')
        DBSession.flush()
        DBSession.close()

        x = self.validator.to_python('asd')

    def test_found_with_slug(self):
        t1 = Thing(name=u'Foo')
        t2 = Thing(name=u'Bar')
        DBSession.flush()
        DBSession.close()

        x = self.slug_validator.to_python('bar-%s' % t2._id)
        assert x
        assert x.name == u'Bar'

    def test_found_slugified_without_slug(self):
        t1 = Thing(name=u'Foo')
        t2 = Thing(name=u'Bar')
        DBSession.flush()
        DBSession.close()

        x = self.slug_validator.to_python('%s' % t2._id)
        assert x
        assert x.name == u'Bar'

    def test_found_slugified_on_objid(self):
        t1 = Thing(name=u'Foo')
        t2 = Thing(name=u'Bar')
        DBSession.flush()
        DBSession.close()

        x = self.slug_validator.to_python(t2._id)
        assert x
        assert x.name == u'Bar'



from tg import abort
import re, unicodedata

def object_primary_key(entity):
    if hasattr(entity, '_id'):
        #mongodb object use _id as the primary key
        return '_id'
    else:
        #multiple primary keys are currently not supported
        return entity.__mapper__.primary_key[0].key

def slugify(entity, value):
    entity_id = getattr(entity, object_primary_key(entity))

    value = unicodedata.normalize('NFKD', value).encode('ascii', 'ignore').decode('ascii')
    value = re.sub('[^\w\s-]', '', value).strip().lower()
    value = re.sub('[-\s]+', '-', value)

    return '%s-%s' % (value, entity_id)

class fail_with(object):
    def __init__(self, code):
        self.code = code

    def __call__(self, *args, **kw):
        abort(self.code)


import tg
import inspect
from utils import object_primary_key

class CacheKey(object):
    """
    Cache Key object, useful to use the entitycached
    decorator when we don't have any real object to pass
    to the cached function
    """
    def __init__(self, cache_key):
        self.cache_key = cache_key

class entitycached(object):
    def __init__(self, key_argument, expire=3600*24*3, cache_type='memory', namespace=None, sqla_merge=False):
        self.cache_key_argument = key_argument
        self.cache_expire = expire
        self.cache_type = cache_type
        self.cache_namespace = namespace
        self.sqla_merge = sqla_merge

    def _get_argspec(self, func):
        try:
            im_func = func.im_func
        except AttributeError:
            im_func = func

        spec = inspect.getargspec(im_func)
        argvals = spec[3]
        if argvals is None:
            argvals = []
        return spec[0], spec[1], spec[2], argvals

    def _get_params_with_argspec(self, args, kw):
        argvars, var_args, argkws, argvals = self.argspec
        if argvars and args:
            kw = kw.copy()
            args_len = len(args)
            for i, var in enumerate(argvars):
                if i >= args_len:
                    break
                kw[var] = args[i]
        return kw

    def _object_primary_key(self, model):
        return object_primary_key(model)

    def _determine_namespace(self, func):
        if self.cache_namespace is not None:
            return self.cache_namespace

        try:
            im_func = func.im_func.__name__
            im_class = (func.im_self if func.im_class == type else func.im_class).__name__
            im_module = self.func_module
        except AttributeError:
            im_func = func.__name__
            im_class = ''
            im_module = self.func_module
        return '%s-%s-%s' % (im_module, im_class, im_func)

    def _determine_cachekey(self, args, kw):
        named_params = self._get_params_with_argspec(args, kw)
        cache_pivot = named_params.get(self.cache_key_argument)

        cache_key = getattr(cache_pivot, 'cache_key', None)
        if not cache_key:
            try:
                object_id = getattr(cache_pivot, self._object_primary_key(cache_pivot))
            except:
                raise ValueError('Unable to determine object primary key, please declare a cache_key property in the object')

            try:
                updated_at = getattr(cache_pivot, 'updated_at').strftime('%Y%m%d%H%M%S')
            except:
                raise ValueError('Object missing an updated_at field, please add one ore declare a cache_key property')

            try:
                cache_key = '%s-%s' % (object_id, updated_at)
            except:
                raise ValueError('Unable to determine object cache key')

        return cache_key

    def _merge_sqla_result(self, results):
        DBSession = tg.config['DBSession']
        return [DBSession.merge(o, load=False) for o in results]

    def __call__(self, func):
        #Precalculate argspec
        if not hasattr(func, 'func_name'):
            raise TypeError('decorated object seems not to be a function or method, \
if you are using multiple decorators check that @entitycached is the first.')

        self.argspec = self._get_argspec(func)
        self.func_module = func.__module__

        def wrapped_function(*args, **kw):
            cache_namespace = self._determine_namespace(func)
            cache_key = self._determine_cachekey(args, kw)

            def call_function():
                return func(*args, **kw)

            cache = tg.cache.get_cache(cache_namespace, type=self.cache_type)
            result = cache.get_value(cache_key, createfunc=call_function, expiretime=self.cache_expire)
            if self.sqla_merge:
                result = self._merge_sqla_result(result)
            return result

        #Look like the decorated function
        wrapped_function.__doc__ = func.__doc__
        wrapped_function.__name__ = func.__name__
        wrapped_function.__entitycached__ = self
        return wrapped_function


from nose.tools import raises
from tgext.datahelpers.caching import entitycached, CacheKey
import ming_base as mingbase
import sqla_base as sqlabase
from datetime import datetime
import tg
from sqlalchemy.orm.exc import DetachedInstanceError

class TestNameSpaceGeneration(object):
    def test_namespace_for_function(self):
        @entitycached('entity')
        def function(entity):
            return 'HI'
        assert function.__entitycached__._determine_namespace(function) == 'tests.test_entitycached--function'

    def test_namespace_for_method(self):
        class Object(object):
            @entitycached('entity')
            def method(self, entity):
                return 'HI'
        o = Object()
        assert o.method.__entitycached__._determine_namespace(o.method) == 'tests.test_entitycached-Object-method'

    def test_namespace_for_staticmethod(self):
        class Object(object):
            @staticmethod
            @entitycached('entity')
            def method(entity):
                return 'HI'
        o = Object()
        assert o.method.__entitycached__._determine_namespace(o.method) == 'tests.test_entitycached--method'

    def test_namespace_for_classmethod(self):
        class Object(object):
            @classmethod
            @entitycached('entity')
            def method(cls, entity):
                return 'HI'
        o = Object()
        assert o.method.__entitycached__._determine_namespace(o.method) == 'tests.test_entitycached-Object-method'

    def test_namespace_forced(self):
        class Object(object):
            @classmethod
            @entitycached('entity', namespace='something')
            def method(cls, entity):
                return 'HI'
        o = Object()
        assert o.method.__entitycached__._determine_namespace(o.method) == 'something'

    @raises(TypeError)
    def test_namespace_for_inverse_classmethod(self):
        class Object(object):
            @entitycached('entity')
            @classmethod
            def method(cls, entity):
                return 'HI'
        o = Object()
        assert o.method.__entitycached__._determine_namespace(o.method) == 'tests.test_entitycached-Object-method'

class TestCacheKey(object):
    def __init__(self):
        @entitycached('entity')
        def function(entity, other):
            return 'HI'
        self.function = function

    def test_cache_key_by_position(self):
        assert self.function.__entitycached__._determine_cachekey([CacheKey(cache_key='HI')], {}) == 'HI'

    def test_cache_key_by_name(self):
        assert self.function.__entitycached__._determine_cachekey([], {'entity':CacheKey(cache_key='HI')}) == 'HI'

    def test_cache_key_varargs(self):
        assert self.function.__entitycached__._determine_cachekey([CacheKey(cache_key='HI')], {}) == 'HI'

    @raises(ValueError)
    def test_cache_key_fail_noid(self):
        class Object(object):
            pass
        self.function.__entitycached__._determine_cachekey([Object()], {})

    @raises(ValueError)
    def test_cache_key_fail_invalidkey(self):
        class Object(object):
            def __init__(self):
                self.updated_at = datetime.now()
                self._id = self
            def __str__(self):
                return None
        self.function.__entitycached__._determine_cachekey([Object()], {})

    def test_cache_key_ming_without_date(self):
        mingbase.setup_database()
        mo = mingbase.Thing(name='something')

        passed = False
        try:
            self.function.__entitycached__._determine_cachekey([mo], {})
        except ValueError:
            passed = True

        mingbase.clear_database()
        assert passed

    def test_cache_key_ming(self):
        mingbase.setup_database()
        mo = mingbase.ThingWithDate(name='something')

        expected_key = '%s-%s' % (mo._id, mo.updated_at.strftime('%Y%m%d%H%M%S'))
        try:
            assert self.function.__entitycached__._determine_cachekey([mo], {}) == expected_key
        finally:
            mingbase.clear_database()

    def test_cache_key_sqla(self):
        sqlabase.setup_database()
        sqlabase.clear_database()

        mo = sqlabase.ThingWithDate(name=u'something')
        sqlabase.DBSession.add(mo)
        sqlabase.DBSession.flush()

        expected_key = '%s-%s' % (mo.uid, mo.updated_at.strftime('%Y%m%d%H%M%S'))
        try:
            assert self.function.__entitycached__._determine_cachekey([mo], {}) == expected_key
        finally:
            sqlabase.clear_database()

class TestCacheWorking(object):
    def __init__(self):
        class Counter(object):
            def __init__(self):
                self.count = 0
            @entitycached('entity')
            def inc(self, entity):
                self.count += 1
                return self.count
        self.counter = Counter()

        class FakeCache(object):
            def __init__(self):
                self._cache = {}
            def get_cache(self, *args, **kw):
                return self
            def get_value(self, key, createfunc, *args, **kw):
                if key not in self._cache:
                    self._cache[key] = createfunc()
                return self._cache[key]
        self.cache = FakeCache()

    def test_cache_works(self):
        tg.cache._push_object(self.cache)
        key = CacheKey(cache_key='HI')
        assert self.counter.inc(key) == 1
        assert self.counter.inc(key) == 1
        assert self.counter.inc(key) == 1
        tg.cache._pop_object(self.cache)

    def test_cache_key_sqlamerge(self):
        sqlabase.setup_database()
        sqlabase.clear_database()

        tg.cache._push_object(self.cache)
        tg.config['DBSession'] = sqlabase.DBSession

        o = sqlabase.Thing(name=u'foobar')
        sqlabase.DBSession.add(o)
        mo = sqlabase.ThingWithDate(name=u'something', related_thing=o)
        sqlabase.DBSession.add(mo)
        sqlabase.DBSession.flush()
        sqlabase.DBSession.commit()

        @entitycached('arg', sqla_merge=True)
        def with_sqla_merge_func(arg):
            return sqlabase.DBSession.query(sqlabase.ThingWithDate).all()

        @entitycached('arg', sqla_merge=False)
        def without_sqla_merge_func(arg):
            return sqlabase.DBSession.query(sqlabase.ThingWithDate).all()

        #Fetch results to cache them
        results = with_sqla_merge_func(CacheKey(cache_key='HI'))
        results = without_sqla_merge_func(CacheKey(cache_key='HI'))

        #remove session
        sqlabase.DBSession.remove()

        #Check that cached results fail to work when not merged back
        try:
            results = without_sqla_merge_func(CacheKey(cache_key='HI'))
            bool(results[0].related_thing.name==u'foobar')
            raise RuntimeError('This should have failed')
        except DetachedInstanceError:
            pass

        #Check that cached results are correctly merged back into session
        results = with_sqla_merge_func(CacheKey(cache_key='HI'))
        assert results[0].related_thing.name==u'foobar'


import tg, os, shutil, cgi, Image, json, tempfile
import sqlalchemy.types as types
import uuid as uuid_m

class AttachedFile(object):
    def __init__(self, file, filename, uuid=None):
        self.uuid = uuid
        if not uuid:
            self.uuid = str(uuid_m.uuid1())

        if isinstance(file, basestring):
            file = open(file)

        self.file = file
        self.filename = unicode(filename)
        self.url = '/'.join([self.attachments_url, self.uuid, self.filename])
        self.local_path = os.path.join(self.attachment_dir, self.filename)

    @property
    def attachments_url(self):
        return tg.config.get('attachments_url', '/attachments')

    @property
    def attachment_dir(self):
        attachments_path = tg.config.get('attachments_path')
        if not attachments_path:
            attachments_path = os.path.join(tg.config['here'], tg.config['package'].__name__.lower(),
                                            'public', 'attachments')
        attachment_dir = os.path.join(attachments_path, self.uuid)
        return unicode(attachment_dir)

    def write(self):
        try:
            os.mkdir(self.attachment_dir)
        except Exception, e:
            pass

        if self.file.name != self.local_path:
            shutil.copyfileobj(self.file, open(self.local_path, 'w+'))
            self.file.seek(0)

    def unlink(self):
        shutil.rmtree(self.attachment_dir)

    def encode(self):
        return unicode(json.dumps({'file':self.local_path, 'filename':self.filename, 'uuid':self.uuid}))

    @classmethod
    def decode(cls, value):
        params = {}
        for key, value in json.loads(value).iteritems():
            params[str(key)] = value
        return cls(**params)

class Attachment(types.TypeDecorator):
    impl = types.Unicode

    def __init__(self, attachment_type=AttachedFile, *args, **kw):
        super(Attachment, self).__init__(*args, **kw)
        self.attachment_type = attachment_type

    def _create_attached_file(self, value):
        if isinstance(value, cgi.FieldStorage):
            filename = value.filename
            file = value.file
        elif isinstance(value, str):
            filename = 'attachment'
            file = tempfile.TemporaryFile()
            file.write(value)
            file.seek(0)
        else:
            filename = os.path.basename(value.name)
            file = value

        return self.attachment_type(file, filename)

    def process_bind_param(self, value, dialect):
        if isinstance(value, cgi.FieldStorage):
            if not bool(getattr(value, 'filename', None)):
                return None
        elif not value:
            return None

        if not isinstance(value, AttachedFile):
            value = self._create_attached_file(value)

        value.write()
        return value.encode()

    def process_result_value(self, value, dialect):
        if not value:
            return None

        return self.attachment_type.decode(value)

class AttachedImage(AttachedFile):
    thumbnail_size = (128, 128)
    thumbnail_format = 'png'

    def __init__(self, file, filename, uuid=None):
        super(AttachedImage, self).__init__(file, filename, uuid)
        
        thumb_filename = 'thumb.'+self.thumbnail_format.lower()
        self.thumb_local_path = os.path.join(self.attachment_dir, thumb_filename)
        self.thumb_url = '/'.join([self.attachments_url, self.uuid, thumb_filename])

    def write(self):
        super(AttachedImage, self).write()

        if self.file.name != self.local_path:
            self.file.seek(0)
            thumbnail = Image.open(self.file)
            thumbnail.thumbnail(self.thumbnail_size, Image.BILINEAR)
            thumbnail = thumbnail.convert('RGBA')
            thumbnail.format = self.thumbnail_format
            thumbnail.save(self.thumb_local_path)



# -*- coding: utf-8 -*-

import tempfile, tg, os, shutil, cgi, base64, Image
from sqlalchemy.schema import Column
from sqlalchemy.types import Unicode, Integer
from sqla_base import setup_database, clear_database, DeclarativeBase, DBSession, Thing
from tgext.datahelpers.fields import Attachment, AttachedFile, AttachedImage

def setup():
    tg.config['attachments_path'] = '/tmp/datahelpers'
    try:
        os.mkdir(tg.config['attachments_path'])
    except:
        pass
    setup_database()

def teardown():
    shutil.rmtree(tg.config['attachments_path'], ignore_errors=True)

class Document(DeclarativeBase):
    __tablename__ = 'docu'

    uid = Column(Integer, autoincrement=True, primary_key=True)
    name = Column(Unicode(16), unique=True)
    content = Column(Attachment)
    photo = Column(Attachment(AttachedImage))

class TestSQLAAttachments(object):
    def __init__(self):
        self.file_content = 'this is the file content'
        self.fake_file = tempfile.NamedTemporaryFile()
        self.fake_file.write(self.file_content)
        self.fake_file.flush()

    def setup(self):
        clear_database()

    def test_create_fromfile(self):
        doc = Document(name=u'Foo')
        doc.content = open(self.fake_file.name)
        DBSession.add(doc)
        DBSession.flush()
        DBSession.commit()

        d = DBSession.query(Document).filter_by(name=u'Foo').first()
        assert d.content.file.read() == self.file_content
        assert d.content.filename == os.path.basename(self.fake_file.name)

    def test_create_fromfield(self):
        field = cgi.FieldStorage()
        field.filename = 'test.jpg'
        field.file = open(self.fake_file.name)

        doc = Document(name=u'Foo', content=field)
        DBSession.add(doc)
        DBSession.flush()
        DBSession.commit()

        d = DBSession.query(Document).filter_by(name=u'Foo').first()
        assert d.content.file.read() == self.file_content
        assert d.content.filename == os.path.basename('test.jpg')
        assert d.content.url.startswith('/attachments')
        assert d.content.url.endswith('test.jpg')

class TestSQLAImageAttachments(object):
    def __init__(self):
        self.file_content = '''R0lGODlhEQAUAPcAAC4uLjAwMDIyMjMzMjQ0NDU1NDY2Njk2Mzg4ODo6Oj49Ozw8PD4+PkE+OEA/PkhAN0tCNk5JPFFGNV1KMFhNNVFHOFJJPVVLPVhOPXZfKXVcK2ZQNGZXNGtZMnNcNHZeNnldMHJfOn1hKXVjOH9oO0BAQEJCQkREREVFREZGRklGQ05KQ0hISEpKSkxMTE5OTlZRSlFQT19XSFBQUFJSUlRUVGFUQmFVQ2ZZQGtdQnNiQqJ/HI1uIYBnLIllKoZrK4FqLoVqL4luLIpsLpt7J515JJ50KZhzLYFnMIFlM4ZlMIFkNI1uNoJoOoVrPIlvO49yMolwPpB2O5p4Op98PaB3IKN4JqN8J6h7I6J5LaZ+LLF+ILGGG7+JG72OGLKEI7aHIrOEJL2JI7mMN76YNcGJG8SOG8WLHMONH86eEs+aFsGSG8eQHMySG9uVFduXFdeeE9eaFdScF96YE9yaFOKcEuOdEtWgFNiiEduhE96pEuqlD+qmD+KpD+yoDu6rDuysDvCuDfCvDeuwD/SzC/a2CvGwDfKxDPi5Cfi5Cvq8CeCjEuehEOagEeijEOKoEOStFMOOK8+TLM6YNNChItGgLtylKt6gMNqgON6jPOChLfi/JOSrNeGvN9KhRtykRNWkSOCnQOCpSOawQue1T+a6Su67SOGsUO/AVAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAACH5BAAAAAAALAAAAAARABQAAAj+AAEIHEiwoMAACBMqXIhQgMOHDwdAfEigYsUCT6KQScNjxwGLBAyINPBhCilUmxIV6uMlw0gEMJeMGWWqFCU8hA4JEgETQYIEDcRw6qRlwgMIGvJwkfAzwYIFXQBB8qHg6VMKFawuYNBBjaIqDhhI+cGgrNmyJXooGrShBJBKcIpwiFCibl0TehDdsWBCiBxLZuIwolMGiwcTJ4gUenThBAokSVRgGFJnzhYQJ1KsyRkkhWfPK87McWPEM4sRhgItCsGitQ5PmtxYUdK6BY4rf/zwYRNmUihRpyQdaUHchQsoX/Y4amTnUqZPoG7YMO7ihfUcYNC0eRMJExMqMKweW59BfkYMEk2ykHAio3x5GvDjy58Pv4b9+/jz2w8IADs='''
        self.file_content = base64.b64decode(self.file_content)
        self.fake_file = tempfile.NamedTemporaryFile()
        self.fake_file.write(self.file_content)
        self.fake_file.flush()

    def setup(self):
        clear_database()

    def test_create_fromfile(self):
        doc = Document(name=u'Foo')
        doc.photo = open(self.fake_file.name)
        DBSession.add(doc)
        DBSession.flush()
        DBSession.commit()

        d = DBSession.query(Document).filter_by(name=u'Foo').first()
        assert d.photo.file.read() == self.file_content
        assert d.photo.filename == os.path.basename(self.fake_file.name)

    def test_create_fromfield(self):
        field = cgi.FieldStorage()
        field.filename = u'àèìòù.gif'
        field.file = open(self.fake_file.name)

        doc = Document(name=u'Foo', photo=field)
        DBSession.add(doc)
        DBSession.flush()
        DBSession.commit()

        d = DBSession.query(Document).filter_by(name=u'Foo').first()
        assert d.photo.file.read() == self.file_content
        assert d.photo.filename == field.filename
        assert d.photo.url.startswith('/attachments/')
        assert d.photo.url.endswith(field.filename)

    def test_thumbnail(self):
        field = cgi.FieldStorage()
        field.filename = 'test.gif'
        field.file = open(self.fake_file.name)

        doc = Document(name=u'Foo', photo=field)
        DBSession.add(doc)
        DBSession.flush()
        DBSession.commit()

        d = DBSession.query(Document).filter_by(name=u'Foo').first()
        assert os.path.exists(d.photo.thumb_local_path)

        thumb = Image.open(d.photo.thumb_local_path)
        thumb.verify()
        assert thumb.format.upper() == d.photo.thumbnail_format.upper()
